@@ -119,9 +119,25 @@ void cancel_drop(gboolean state)
     drop_cancelled = state;
 }
 
-static void clear_selection_list(void)
+void reselect_dnd_list(GtkTreeView *treeview){
+    GList *tmp;
+    GtkTreeSelection *selection = gtk_tree_view_get_selection(treeview);
+    for (tmp=selection_list;tmp;tmp=tmp->next){
+	selection_list_t *sl=tmp->data;
+        if(sl && gtk_tree_row_reference_valid(sl->reference)){
+           GtkTreePath *treepath = gtk_tree_row_reference_get_path(sl->reference);
+	   /*printf("DBG: reselecting %s\n",sl->en->path);*/
+	   gtk_tree_selection_select_path (selection,treepath);
+	   gtk_tree_path_free(treepath);
+	}
+    }
+}
+
+void clear_dnd_selection_list(void)
 {
     GList *tmp = selection_list;
+    if (dragging) return;
+    /*printf("DBG: clearing selection list!\n");*/
     while(tmp)
     {
 	selection_list_t *sl;
@@ -129,6 +145,7 @@ static void clear_selection_list(void)
 	if(sl->reference)
 	    gtk_tree_row_reference_free(sl->reference);
 	g_free(sl);
+	sl=NULL;
 	tmp = tmp->next;
     }
     if(selection_list)
@@ -140,6 +157,12 @@ static void clear_selection_list(void)
     selection_list = NULL;
 }
 
+void   on_drag_begin(GtkWidget *widget,GdkDragContext *drag_context,gpointer user_data){
+    	dragging = TRUE;	
+	/* this works better with plain motion-event */
+	/*reselect_dnd_list((GtkTreeView *)widget);   */
+}
+
 static gboolean valid_drop_site(GtkTreeView * treeview, gint x, gint y, tree_entry_t ** en, GtkTreeRowReference ** reference)
 {
     tree_details_t *tree_details = get_tree_details(treeview);
@@ -224,7 +247,7 @@ static gboolean valid_drop_site(GtkTreeView * treeview, gint x, gint y, tree_ent
 	if(!gtk_tree_model_iter_parent(treemodel, &parent, &iter))
 	    goto nodrop;
 	gtk_tree_model_get(treemodel, &parent, ENTRY_COLUMN, en, -1);
-	if(!en) goto nodrop;
+	if(!(*en)) goto nodrop;
 	if(IS_BOOKMARK_TYPE((*en)->type) && IS_ROOT_TYPE((*en)->type))
 	    goto drop_ok;
 	if(!IS_DIR((*en)->type) && 
@@ -275,6 +298,7 @@ char  *CreateSMBTmpList(GtkTreeView *treeview,
 			 fclose(tmpfile);
 			 unlink (fname);
 			 g_free(w);
+			 w=NULL;
 			 return NULL;
 		 } 
 		 if (S_ISREG(s.st_mode)) {
@@ -297,6 +321,7 @@ char  *CreateSMBTmpList(GtkTreeView *treeview,
 	         }
 		 fflush(NULL);
 		 g_free(w);
+		 w=NULL;
     }
     fprintf(tmpfile,"ls;\n");
     fclose (tmpfile);
@@ -342,8 +367,10 @@ void on_drag_data(GtkWidget * widget, GdkDragContext * context, gint x, gint y,
 	goto drag_over;
     }
 
-    if(src_host)
+    if(src_host){
 	g_free(src_host);
+	src_host=NULL;
+    }
     atomo = XInternAtom(GDK_DISPLAY(), "WM_CLIENT_MACHINE", FALSE);
     {
 	unsigned char *property_data;
@@ -465,6 +492,7 @@ void on_drag_data(GtkWidget * widget, GdkDragContext * context, gint x, gint y,
       tmpfile=CreateSMBTmpList(treeview,list,target,
 		      IS_SAMBA_SERVER(en->subtype));
       g_free(target);
+      target=NULL;
       if (!tmpfile) {
          /*fprintf(stderr,"dbg:null tmpfile\n");*/
          list=uri_free_list (list);
@@ -481,16 +509,28 @@ void on_drag_data(GtkWidget * widget, GdkDragContext * context, gint x, gint y,
     {
 	for(tmp = list; tmp != NULL; tmp = tmp->next)
 	{  
-	    char *argv[3];
+	    gchar *in_cmd;
+	    
 	    u = tmp->data;
-	    print_status(treeview, "xf_INFO_ICON", 
+	    
+	    if (en->filter) {
+		in_cmd = g_strconcat(en->path," ",en->filter,NULL);
+	    	print_status(treeview, "xf_INFO_ICON", 
+			       _("Executing"), " : ", 
+			        FILENAME(en)," ",en->filter,
+			       NULL);
+	    }
+	    else {
+		in_cmd = g_strdup(en->path);
+	    	print_status(treeview, "xf_INFO_ICON", 
 			       _("Executing"), " : ", 
 			        FILENAME(en),
 			       NULL);
-	    argv[0]=en->path;
-	    argv[1]=u->url;
-	    argv[2]=0;
-	    runv(treeview, argv);
+	    }
+	    
+ 	    on_run_path(treeview, in_cmd, u->url, IS_IN_TERM(en->subtype),
+			   FALSE, FALSE);
+	    g_free(in_cmd);
 	}
 	list = uri_free_list(list);
 	goto drag_over;
@@ -520,6 +560,7 @@ void on_drag_data(GtkWidget * widget, GdkDragContext * context, gint x, gint y,
 	if(tree_details->preferences & RSYNC_X_SCP)
 	    rsync(treeview, srcs, en->path);
 	g_free(srcs);
+	srcs=NULL;
 
 #if 0
 	for(tmp = list; tmp != NULL; tmp = tmp->next)
@@ -558,11 +599,13 @@ void on_drag_data(GtkWidget * widget, GdkDragContext * context, gint x, gint y,
 	    {
 		list = uri_free_list(list);
 		g_free(p);
+		p=NULL;
 		print_status(treeview, "xf_ERROR_ICON", _("Source and target are the same!"), NULL);
 		/*fprintf(stderr,"dbg:nonsense 2\n"); */
 		goto drag_over;
 	    }
 	    g_free(p);
+	    p=NULL;
 	}
     }
 
@@ -632,27 +675,42 @@ void on_drag_data(GtkWidget * widget, GdkDragContext * context, gint x, gint y,
     if (!IS_BOOKMARK_TYPE(en->type)) local_monitor(treeview,TRUE);
 }
 
+gboolean is_in_dnd_selection(GtkTreePath * path){
+	GList *tmp;
+	if (!path) return TRUE;
+	for (tmp=selection_list;tmp;tmp=tmp->next){
+		selection_list_t *sl=tmp->data;
+           	GtkTreePath *treepath = gtk_tree_row_reference_get_path(sl->reference);
+		if(gtk_tree_path_compare (path,treepath)==0){
+	           gtk_tree_path_free(treepath);
+		   return TRUE;
+		}
+	        gtk_tree_path_free(treepath);
+	}
+	return FALSE;
+}
+
 void get_dnd_selection(GtkTreeModel * treemodel, GtkTreePath * path, GtkTreeIter * iter, gpointer data)
 {
     GtkTreeView *treeview = (GtkTreeView *) data;
     GtkTreeSelection *selection = gtk_tree_view_get_selection(treeview);
     tree_entry_t *en;
 
-    if (drag_type==DRAG_TYPE_INCONSISTENT) return;
+   /* if (drag_type==DRAG_TYPE_INCONSISTENT) return;*/
 
     gtk_tree_model_get(treemodel, iter, ENTRY_COLUMN, &en, -1);
-    if (!IS_LOCAL_TYPE(en->type) && IS_ROOT_TYPE(en->type)) return;
     if(!IS_PATH(en->type) && !IS_NETTHING(en->subtype) )
     {
 	gtk_tree_selection_unselect_iter(selection, iter);
 	return;
     }
-    if((IS_PATH(en->type) && drag_type==DRAG_TYPE_NET) ||
+    if (!IS_LOCAL_TYPE(en->type) && IS_ROOT_TYPE(en->type)) return;
+    /*if((IS_PATH(en->type) && drag_type==DRAG_TYPE_NET) ||
        (IS_NETTHING(en->subtype) && drag_type==DRAG_TYPE_LOCAL)){
         drag_type=DRAG_TYPE_INCONSISTENT;
 	print_diagnostics(treeview,"xf_ERROR_ICON",strerror(E2BIG),"\n",NULL); 
 	return;
-    }
+    }*/
     if(IS_PATH(en->type)) drag_type=DRAG_TYPE_LOCAL;
     if(IS_NETTHING(en->subtype)) drag_type=DRAG_TYPE_NET;
 
@@ -663,7 +721,8 @@ void get_dnd_selection(GtkTreeModel * treemodel, GtkTreePath * path, GtkTreeIter
 	    g_assert_not_reached();
 	sl->en = en;
 	sl->reference = gtk_tree_row_reference_new(treemodel, path);
-	selection_list = g_list_append(selection_list, sl);
+	selection_list = g_list_prepend(selection_list, sl);
+	/*printf("DBG: added to selection list=%s\n",en->path);*/
     }
     selection_len += (strlen(en->path) + 7);
 
@@ -684,28 +743,27 @@ void on_drag_data_get(GtkWidget * widget, GdkDragContext * context, GtkSelection
     GtkTreeView *treeview = (GtkTreeView *) widget;
     GtkTreeSelection *selection = gtk_tree_view_get_selection(treeview);
     
-    if(dragging) return;
-    dragging = TRUE;
-
     if(!widget)	return;
-    if(selection_list)	clear_selection_list();
-
+    /*if(selection_list)	clear_dnd_selection_list();*/
     
-    drag_type=DRAG_TYPE_UNDEFINED;
-    gtk_tree_selection_selected_foreach(selection, get_dnd_selection, (gpointer) treeview);
-    if(!selection_list || !(drag_type&(DRAG_TYPE_LOCAL|DRAG_TYPE_NET))) 
+    /*gtk_tree_selection_selected_foreach(selection, get_dnd_selection, (gpointer) treeview);*/
+    /*drag_type=DRAG_TYPE_UNDEFINED;*/
+    if(!selection_list) return;
+   
+    if(!(drag_type&(DRAG_TYPE_LOCAL|DRAG_TYPE_NET))) 
 	    return;
 
 
-
-
     /* prepare data for the receiver */
     switch (info)
     {
 	case TARGET_ROOTWIN:	/* not implemented (yet) */
 	 return;
 	default:
-	 if(dnd_data)g_free(dnd_data);
+	 if(dnd_data){
+		 g_free(dnd_data);
+		 dnd_data=NULL;
+	 }
 	 switch (drag_type){
 	  default:
 		  g_assert_not_reached();
@@ -716,6 +774,7 @@ void on_drag_data_get(GtkWidget * widget, GdkDragContext * context, GtkSelection
 	    {
 		selection_list_t *sl;
 		sl = (selection_list_t *) tmp->data;
+		if (!gtk_tree_row_reference_valid(sl->reference)) return;
 		if(strncmp(sl->en->path, "tar:", strlen("tar:")) == 0)
 		{
 		    sprintf(files, "%s\r\n", sl->en->path);
@@ -734,6 +793,7 @@ void on_drag_data_get(GtkWidget * widget, GdkDragContext * context, GtkSelection
 	        int addlen;
 		selection_list_t *sl;
 		sl = (selection_list_t *) tmp->data;
+		if (!gtk_tree_row_reference_valid(sl->reference)) return;
 		addlen=strlen("smb://@://\r\n");
 		addlen += (strlen(sl->en->path));
 		addlen += ((sl->en->tag)?strlen(sl->en->tag):strlen("GUEST%%"));
@@ -748,6 +808,7 @@ void on_drag_data_get(GtkWidget * widget, GdkDragContext * context, GtkSelection
 		selection_list_t *sl;
 		char *server,*remote_file;
 		sl = (selection_list_t *) tmp->data;
+		if (!gtk_tree_row_reference_valid(sl->reference)) return;
 		server=g_strdup(sl->en->path+2);
 		strtok(server,"/");
 		if (IS_XF_NETWS(sl->en->subtype)) {
@@ -764,6 +825,7 @@ void on_drag_data_get(GtkWidget * widget, GdkDragContext * context, GtkSelection
 			  (IS_NETDIR(sl->en->subtype))?"/\r\n":"\r\n");
 		}
 		g_free(server);
+		server=NULL;
 	        files = files + strlen(files);
 	    }
 	    break;
@@ -776,7 +838,8 @@ void on_drag_data_get(GtkWidget * widget, GdkDragContext * context, GtkSelection
 
     gtk_selection_data_set(selection_data, selection_data->target, 
 		    8, (const guchar *)dnd_data, selection_len); 
-
+    dragging=FALSE;
+    /*printf("DBG drag get done\n");    */
 }
 
 gboolean on_drag_motion(GtkWidget * widget, GdkDragContext * dc, gint x, gint y, guint t, gpointer data)
@@ -833,10 +896,11 @@ void on_drag_data_delete(GtkWidget * widget, GdkDragContext * context, gpointer
 
 void on_drag_end(GtkWidget * widget, GdkDragContext * context, gpointer data)
 {
-    GList *tmp = selection_list;
+    GList *tmp;
     GtkTreeView *treeview = (GtkTreeView *) widget;
     tree_details_t *tree_details = get_tree_details(treeview);
 
+    /*printf("DBG drag end\n");*/
     dragging = FALSE;
     if(!widget)
 	return;
@@ -852,23 +916,30 @@ void on_drag_end(GtkWidget * widget, GdkDragContext * context, gpointer data)
 	printf("DBG: cannot set tree_details->loading! (on_drag_end())\n");
 	return;
     }
-    while(tmp)
+    for (tmp=selection_list;tmp;tmp=tmp->next)
     {
 	selection_list_t *sl;
 	sl = (selection_list_t *) tmp->data;
-	if(sl->reference){
+	if(sl->reference && gtk_tree_row_reference_valid(sl->reference)){
 		update_dir(treeview, sl->reference);
 	}
-	g_free(sl);
-	tmp = tmp->next;
+    }
+
+    /* this would unselect selection after drop: */
+#if 0
+    for (tmp=selection_list;tmp;tmp=tmp->next)
+    {
+	g_free(tmp->data);
     }
     if(selection_list)
     {
 	g_list_free(selection_list);
-	selection_list = NULL;
 	selection_len = 0;
     }
     selection_list = NULL;
+#endif
+
+    
     unset_load_wait(&tree_details);
     return;
 }
@@ -879,6 +950,7 @@ void setup_drag_signal(GtkTreeView * treeview)
     g_signal_connect(G_OBJECT(treeview), "drag_data_get", G_CALLBACK(on_drag_data_get), NULL);
     g_signal_connect(G_OBJECT(treeview), "drag_motion", G_CALLBACK(on_drag_motion), NULL);
     g_signal_connect(G_OBJECT(treeview), "drag_end", G_CALLBACK(on_drag_end), NULL);
+    g_signal_connect(G_OBJECT(treeview), "drag_begin", G_CALLBACK(on_drag_begin), NULL);
 #if 0
     this signal looks bugged in gtk.only works well forsame widget.segv otherwise g_signal_connect(G_OBJECT(treeview), "drag_data_delete", G_CALLBACK(on_drag_data_delete), NULL);
 #endif
