@@ -83,11 +83,24 @@
 #include "icons.h"
 #include "treeview.h"
 
+
 extern root_t root[];
 
 static gboolean diagnostics_disabled=FALSE;
 extern gboolean double_treeview;
 
+void count_selection(GtkTreeModel * treemodel, 
+		GtkTreePath * treepath, 
+		GtkTreeIter * iter, 
+		gpointer data)
+{
+    GtkTreeView *treeview = (GtkTreeView *) data;
+    tree_details_t *tree_details = get_tree_details(treeview);
+    tree_details->selectionOK++;
+    return ;
+}
+
+#if 0
 static gboolean correct_selection;
 static void query_selection(GtkTreeModel * model, GtkTreePath * path, GtkTreeIter * iter, gpointer data)
 {
@@ -97,6 +110,7 @@ static void query_selection(GtkTreeModel * model, GtkTreePath * path, GtkTreeIte
     return;
 }
 
+
 GtkTreeView *get_selected_treeview(GtkWidget *w){
     GtkTreeView *treeview;
     GtkTreeSelection *selection;
@@ -113,12 +127,28 @@ GtkTreeView *get_selected_treeview(GtkWidget *w){
     }
     return (GtkTreeView *)lookup_widget(w,"treeview");	
 }
+#endif
 
 /* this function is to workaround a gtk bug in treeviews, where a 
  * while(gtk_events_pending()) will not break... 
- * Let's say 256 iterations suffice... */
-void process_pending_gtk(){
+ * Let's say 256 iterations suffice... 
+ * But not always (if called from a timeout function),
+ * so let's provide a blocking mechanism */
+static gboolean pending_busy=FALSE;
+gboolean processing_pending(void){
+	return pending_busy;
+}
+
+void set_processing_pending(void){
+	pending_busy=TRUE;
+}
+void unset_processing_pending(void){
+	pending_busy=FALSE;
+}
+	
+void process_pending_gtk(void){
   int i;
+  if (processing_pending()) return;
   for (i=0;i<256;i++){
 	  if (!gtk_events_pending()) break;
 	  else gtk_main_iteration();
@@ -475,11 +505,14 @@ char *abreviate(char *path)
     int i;
     static char *label = NULL;
 
+    if (!path) return "";
     if(strlen(path) <= 21)
 	return path;
 
-    if(label)
+    if(label){
 	g_free(label);
+	label=NULL;
+    }
     label = (char *)malloc(strlen(path) + 1);
     if(!label)
 	return path;
@@ -609,29 +642,54 @@ void cursor_reset(GtkTreeView * treeview)
 
 }
 
-#if 0
 void set_font(GtkTreeView * treeview, GtkTreeIter * iterator)
 {
+#if 10
     PangoFontDescription *desc;
     int size;
     GtkTreeModel *treemodel = gtk_tree_view_get_model(treeview);
     tree_entry_t *en = get_entry(treeview, iterator);
 
+#if 0
+    /* this does not work, probably because there is a theme running...*/
+    /* so we will probably have to do the color business by inserting
+     * tags and doing a pango_layout+attributes+attributelist stuff,
+     * however that may be done... We keep the column to store the
+     * color value for the tag creation...*/
+    static GdkColormap *cmap=NULL;
+    GdkColor t_color;
+    t_color.pixel = 0;
+    t_color.red = 0;
+    t_color.green = 0xff;
+    t_color.blue = 0;
+
+    if (!cmap) {
+	    cmap=gdk_colormap_new (gdk_visual_get_system(),TRUE);
+	    gdk_colormap_alloc_color (cmap,
+			    &t_color,TRUE,TRUE);
+    }
+	    
+    
+    gtk_tree_store_set((GtkTreeStore *) treemodel, iterator, 
+		    COLOR_COLUMN, t_color, -1);
+#endif
+    
     if(!en)
 	return;
-
-    if(IS_ROOT_TYPE(en->type))
+#if 0
+    if(!IS_ROOT_TYPE(en->type))
     {
 	if(IS_SIZE_XL(en->type))
-	    size = 18 * 1000;
-	else if(IS_SIZE_L(en->type))
-	    size = 16 * 1000;
-	else if(IS_SIZE_M(en->type))
 	    size = 14 * 1000;
-	else
+	else if(IS_SIZE_L(en->type))
 	    size = 12 * 1000;
+	else if(IS_SIZE_M(en->type))
+	    size = 10 * 1000;
+	else
+	    size = 8 * 1000;
     }
     else
+#endif
     {
 	if(IS_SIZE_XL(en->type))
 	    size = 16 * 1000;
@@ -645,7 +703,7 @@ void set_font(GtkTreeView * treeview, GtkTreeIter * iterator)
 
     desc = pango_font_description_new();
 
-#if 10
+#if 0
     if(IS_ROOT_TYPE(en->type))
 	pango_font_description_set_family(desc, "times");
 #endif
@@ -656,8 +714,8 @@ void set_font(GtkTreeView * treeview, GtkTreeIter * iterator)
 
     gtk_tree_store_set((GtkTreeStore *) treemodel, iterator, FONT_COLUMN, desc, -1);
     pango_font_description_free(desc);
-}
 #endif
+}
 
 tree_details_t *get_tree_details(GtkTreeView * treeview)
 {
@@ -668,9 +726,26 @@ tree_details_t *get_tree_details(GtkTreeView * treeview)
 
 GtkTreeView *get_treeview(GtkWidget * widget)
 {
-    if(!widget)
+    GtkTreeView *treeview;
+    tree_details_t *tree_details;
+    GtkTreeSelection *selection;
+    if(!widget) {
+	printf("DBG:get_treeview() got null argument!\n");
 	return NULL;
-    return (GtkTreeView *) lookup_widget(widget, "treeview");
+    }
+    treeview = (GtkTreeView *) lookup_widget(widget, "treeview2");
+    tree_details = g_object_get_data(G_OBJECT(treeview), "tree_details");
+    selection = gtk_tree_view_get_selection(treeview);
+    tree_details->selectionOK=0;
+    gtk_tree_selection_selected_foreach(selection,count_selection, (gpointer) treeview);
+    if (tree_details->selectionOK==0)
+	    treeview = (GtkTreeView *) lookup_widget(widget, "treeview");
+    
+    return treeview;
+}
+
+GtkTreeView *get_selected_treeview(GtkWidget * widget){
+	return get_treeview(widget);
 }
 
 
@@ -738,12 +813,16 @@ gchar *utf_2_local_string(char *t){
     printf("DBG: codeset for system is: %s\n",from_codeset);    
 #endif
     
-    if(s) g_free(s);
+    if(s) {
+	    g_free(s);
+	    s=NULL;
+    }
 #ifdef DEBUG    
     printf("DBG: setting to local codeset-> %s\n",t); 
 #endif
     s = g_convert (t,strlen(t),from_codeset,"UTF-8",&r_bytes, &w_bytes, &error);
     g_free(from_codeset);
+    from_codeset=NULL;
 
     if(error){
 	g_error_free(error);
@@ -791,7 +870,10 @@ gchar *my_utf_string(char *t)
     printf("DBG: codeset for system is: %s\n",from_codeset);    
 #endif
     
-    if(s) g_free(s);
+    if(s) {
+	    g_free(s);
+	    s=NULL;
+    }
 #ifdef DEBUG    
     printf("DBG: utfing %s\n",t); 
 #endif
@@ -800,6 +882,7 @@ gchar *my_utf_string(char *t)
 	    *c = ' ';
     s = g_convert (t,strlen(t),"UTF-8",from_codeset,&r_bytes, &w_bytes, &error);
     g_free(from_codeset);
+    from_codeset=NULL;
 
     if(!s)
     {
@@ -843,12 +926,16 @@ gchar * my_cut_utf_string(gchar *s){
 		if (is_number(getenv("XFFM_STATUS_LINE_LENGTH")))
 		   max_len=(atoi(getenv("XFFM_STATUS_LINE_LENGTH")));
 	}
-	if (u) g_free(u);
+	if (u) {
+		g_free(u);
+		u=NULL;
+	}
         if (g_utf8_strlen(s,-1) > max_len){
 	    u=g_utf8_offset_to_pointer (v,max_len-4);
 	    *u=0;
 	    u=g_strjoin ("",v,"...",NULL);
 	    g_free(v);
+	    v=NULL;
        } else u=v;
        return u;
 } 
@@ -1015,9 +1102,308 @@ void hide_stop(GtkWidget * widget)
 
 /*XXX put these separate lib */
 
+#define MAX_COMBO_ELEMENTS 13
+
+static void clean_history_list(GList **list){
+	GList *tmp;
+	if (!*list) return ;
+	for (tmp=*list;tmp;tmp=tmp->next){
+		/*printf("freeing %s\n",(char *)tmp->data);*/
+		g_free(tmp->data);
+		tmp->data=NULL;
+	}
+	g_list_free(*list);
+	*list=NULL;
+	return;
+}
+
+/*summary:
+ * control-tab moves to next slash or end of string
+ * tab cycles to the next history element that matches
+ * control-del removes the item from the history
+ * */
+
+void set_limited_combo(history_combo_info_t *combo_info, char *token){
+    int count;
+    GList *tmp,*oldlist;
+    
+    if (!combo_info->list || !combo_info->active_dbh_file) {
+	    return;
+    }
+    oldlist=combo_info->limited_list;
+    combo_info->limited_list=NULL;
+#ifdef DEBUG
+    if (token) printf("DBG: token=%s\n",token);
+#endif
+    for (tmp=combo_info->list,count=0;tmp;tmp=tmp->next){
+	gchar *p=(gchar *)tmp->data;
+	if (!p) continue;
+#ifdef DEBUG
+         if (token) printf("DBG: token=%s <-->%s\n",token,p);
+#endif
+	if (!token || strncmp(token,p,strlen(token))==0){
+		combo_info->limited_list=g_list_append(combo_info->limited_list,g_strdup(p));
+		if (++count >= MAX_COMBO_ELEMENTS) break;
+	}
+    }
+
+    if (combo_info->limited_list) {
+	    gtk_combo_set_popdown_strings(GTK_COMBO(combo_info->combo),combo_info->limited_list);
+    	    clean_history_list(&oldlist);
+    } else {
+#ifdef DEBUG
+	    printf("DBG: list is null!\n");
+#endif
+	    combo_info->limited_list=oldlist;
+    }
+}
+
+
+gint on_key_press_history(GtkWidget * entry, GdkEventKey * event, gpointer data)
+{
+    gchar *text[2]={NULL,NULL};
+    gchar c[]={0,0};
+    gchar *fulltext=NULL;
+    history_combo_info_t *combo_info = (history_combo_info_t *)data;
+    GtkTreeView *treeview = combo_info->treeview;
+    GList *tmp=combo_info->list;
+    GtkCombo *combo;
+    GtkEditable *editable=(GtkEditable *)entry;
+    gint pos1,pos2,pos;
+    gboolean preselection=
+	    gtk_editable_get_selection_bounds (editable,&pos1,&pos2);
+
+    pos=gtk_editable_get_position(editable);
+     
+    if (!preselection) pos1 =  pos2 = -1;
+    
+
+    /*printf("DBG(2):pos= %d\n",pos);*/
+    /*printf("DBG(2):got key= 0x%x\n",event->keyval);*/
+    /*printf("DBG(2):dbhfile= %s\n",combo_info->active_dbh_file);*/
+
+ if (event->keyval == GDK_KP_Down && event->state==GDK_MOD1_MASK){
+	 return FALSE;
+ }
+ if (event->keyval == GDK_KP_Up && event->state==GDK_MOD1_MASK){
+	 return FALSE;
+ }
+ if (event->keyval == GDK_Down && event->state==GDK_MOD1_MASK){
+	 return FALSE;
+ }
+ if (event->keyval == GDK_Up && event->state==GDK_MOD1_MASK){
+	 return FALSE;
+ }
+    
+    
+ if (event->keyval == GDK_BackSpace && (event->state&GDK_CONTROL_MASK)){
+	gtk_editable_delete_text (editable,0,-1);
+	return TRUE;
+ }
+
+ if (event->keyval == GDK_Tab) {
+    gint start,end;
+    {	
+    	if (gtk_editable_get_selection_bounds(editable,&start,&end)){
+	  gchar *p;
+	  p=gtk_editable_get_chars (editable,start,end);
+	  if (*p =='/' || *p ==' ') start++;
+	  g_free(p);p=NULL;	  
+	/*printf("selection at %d %d\n",start,end);*/
+	  for (;start<=end;start++){
+		p=gtk_editable_get_chars (editable,start,end);
+		/*printf("start=%d got char %s\n",start,p);*/
+		if (*p =='/' || *p ==' ') {
+			gtk_editable_delete_text (editable,start,end);
+			g_free(p);p=NULL;
+			break;
+		}
+		g_free(p);p=NULL;
+	  }
+   	  gtk_editable_select_region (editable, 0,0);
+    	  gtk_editable_set_position(editable,-1);
+	  
+	}
+        return TRUE;
+    }
+ }
+
+
+ /* construct search string... (limited to 128 bytes)*/
+
+ /* control-delete will remove the selected item from the
+  * history dbh file (to remove stale history items) */
+ else if (event->keyval == GDK_Right || event->keyval == GDK_Left) {
+	 if (event->keyval == GDK_Right){
+		 /*printf ("DBG: right\n");*/
+		 gtk_editable_set_position (editable,pos+1);
+	 }else{
+		 /*printf ("DBG: left\n");*/
+		 if (pos-1 >= 0) gtk_editable_set_position (editable,pos-1);
+	 }
+	return TRUE;	 
+ }
+
+ else if (event->keyval == GDK_BackSpace ||
+     event->keyval == GDK_Delete ||
+     event->keyval == GDK_KP_Delete ||
+     event->keyval == GDK_KP_Space ||
+/*     (event->keyval >= GDK_KP_Delete && event->keyval <= GDK_KP_9) ||*/
+     (event->keyval >= GDK_KP_0 && event->keyval <= GDK_KP_9) ||
+     (event->keyval >= GDK_space && event->keyval <= GDK_asciitilde)) {
+	/* get entry text */
+
+   	if (event->keyval == GDK_BackSpace ){
+    	    if (preselection) {
+	        gtk_editable_delete_text (editable,pos1,-1);
+		return TRUE;
+	    }
+	    if (pos==0) return TRUE;
+      	    text[0]=gtk_editable_get_chars (editable,0,pos-1);
+	    text[1]=gtk_editable_get_chars (editable,pos,-1);
+	    fulltext=g_strconcat(text[0],text[1],NULL);
+   	    g_free(text[0]);
+    	    g_free(text[1]);
+	    text[0]=text[1]=NULL;
+	    gtk_editable_delete_text (editable,0,-1);
+	    pos1=0;
+	    gtk_editable_insert_text (editable,(const gchar *)fulltext,
+			     strlen(fulltext),&pos1);
+	    gtk_editable_set_position (editable,pos-1);
+	    /*printf("DBG: inserting %s\n",fulltext);*/
+	    g_free(fulltext); 
+	    fulltext=NULL;
+	    return TRUE;
+	} 
+	else if (event->keyval == GDK_Delete ||
+     	    event->keyval == GDK_KP_Delete){
+	  if (combo_info->active_dbh_file && event->state&GDK_CONTROL_MASK){ /* remove stale entries */
+    	    fulltext=gtk_editable_get_chars (editable,0,-1);
+	    remove_from_history(combo_info->active_dbh_file,fulltext);
+	    print_diagnostics(treeview,"xf_INFO_ICON",
+			    _("Variable")," \"",fulltext,"\" ",
+			    _("Cancelled"),"\n",NULL);
+	    g_free(fulltext);
+	    fulltext=NULL;
+	    cancel_input(treeview);
+	    return TRUE;
+	  } else {
+		  /*printf("DBG: pos=%d, pos1=%d\n",pos,pos1);*/
+	    g_free(fulltext);
+	    fulltext=NULL;
+    	    if (preselection) {
+	        gtk_editable_delete_text (editable,pos1,-1);
+		return TRUE;
+	    }
+      	    text[0]=gtk_editable_get_chars (editable,0,pos);
+	    text[1]=gtk_editable_get_chars (editable,pos+1,-1);
+	    fulltext=g_strconcat(text[0],text[1],NULL);
+   	    g_free(text[0]);
+    	    g_free(text[1]);
+	    text[0]=text[1]=NULL;
+	    
+	    gtk_editable_delete_text (editable,0,-1);
+	    pos1=0;
+	    gtk_editable_insert_text (editable,(const gchar *)fulltext,
+			     strlen(fulltext),&pos1);
+	    gtk_editable_set_position (editable,pos);
+	    /*printf("DBG: inserting %s\n",fulltext);*/
+	    g_free(fulltext); 
+	    fulltext=NULL;
+	    return TRUE;
+	  }
+	} else {
+	  *c=event->keyval;	
+    	  if (preselection) {
+	        gtk_editable_delete_text (editable,pos1,-1);
+      	        text[0]=gtk_editable_get_chars (editable,0,-1);
+	  	fulltext=g_strconcat(text[0],c,NULL);
+		text[1]=NULL;
+	  	pos=0;
+	        gtk_editable_delete_text (editable,0,-1);
+	  	gtk_editable_insert_text (editable,(const gchar *)fulltext,
+			     strlen(fulltext),&pos);
+	  	gtk_editable_set_position (editable,pos);
+	  } else {
+		  /*printf("DBG: pos=%d\n",pos);*/
+      	        text[0]=gtk_editable_get_chars (editable,0,pos);
+    	        text[1]=gtk_editable_get_chars (editable,pos,-1);
+	  	fulltext=g_strconcat(text[0],c,text[1],NULL);
+		  /*printf("DBG: pos=%d fulltext=%s\n",pos,fulltext);*/
+	  	pos1=0;
+	        gtk_editable_delete_text (editable,0,-1);
+	  	gtk_editable_insert_text (editable,(const gchar *)fulltext,
+			     strlen(fulltext),&pos1);
+	  	gtk_editable_set_position (editable,pos+1);
+	  }
+	  
+	}		
+    	g_free(text[0]);
+    	g_free(text[1]);
+	text[0]=text[1]=NULL;
+ }
+ else return FALSE;
+
+ if (combo_info->combo) {
+	 /*printf("DBG: setting limited list and emitting signal...\n");*/
+	 set_limited_combo(combo_info,fulltext);
+#if 0
+/*	 XXX: activate the combo but dont lose keyboard focus on the entry:
+ *	 the following code should work to activate the combo, but causes
+ *	 a crash because the g_signal_emit_by_name is not correctly called
+ *	 signal should have form "signal::detail"*/
+	 {
+	   GdkEventKey event;	 
+	   event.state = gtk_accelerator_get_default_mod_mask ();
+	   event.keyval = GDK_KP_Down;
+	   g_signal_emit_by_name(combo_info->entry,"key_press_event",
+			 &event,
+			 (gpointer)(combo_info->combo));
+	 }
+#endif
+ }
+ 
+ if (fulltext){
+    /* look for in ordered GList*/
+    /*printf("DBG:fulltext is %s\n",fulltext);*/
+    for (;tmp;tmp=tmp->next){
+	gchar *p=(gchar *)tmp->data;
+	if (!p) continue;
+        /*printf("DBG:compare with-> %s\n",p);*/
+	if (strncmp(fulltext,p,strlen(fulltext))==0){
+	    gchar *to_write=p+strlen(fulltext);
+	    /*printf("DBG: gotcha %s->%s, to write=%s\n",fulltext,p,to_write);*/
+            /* if found, complete with untyped part selected.*/
+	    gtk_editable_delete_text (editable,0,-1);
+	    pos1=0;
+	    gtk_editable_insert_text (editable,(const gchar *)fulltext,
+			     strlen(fulltext),&pos1);
+	    pos2=pos1;
+	    gtk_editable_insert_text (editable,(const gchar *)to_write,
+			     strlen(to_write),&pos2);
+    	    gtk_editable_select_region (GTK_EDITABLE (entry), pos1, -1);
+    	    g_free(fulltext);
+	    fulltext=NULL;
+	    /* nogoood: g_signal_emit_by_name ((gpointer)(combo_info->combo),"activate");*/
+	    return TRUE;
+	}
+    }
+    g_free(fulltext);
+    fulltext=NULL;
+ }
+ /*if (event->keyval == GDK_Tab || 
+	event->keyval == GDK_Delete ||
+	event->keyval == GDK_BackSpace ||
+     	event->keyval == GDK_KP_Delete) 
+   return TRUE;*/
+			    
+ return (TRUE);
+}
+
 
 #define HISTORY_ITEMS 13
 static GList **the_list;
+static time_t last_hit;
 
 typedef struct history_dbh_t{
 	int hits;
@@ -1025,41 +1411,49 @@ typedef struct history_dbh_t{
 	char path[256];
 }history_dbh_t;
 
+
 static int history_compare (gconstpointer a,gconstpointer b){
 	history_dbh_t *da=(history_dbh_t *)a;
 	history_dbh_t *db=(history_dbh_t *)b;
-	if (db->hits != da->hits) return db->hits - da->hits;
-	if (db->last_hit != da->last_hit) return db->last_hit - da->last_hit;
+	/*if (db->last_hit >= last_hit) last_hit = db->last_hit;
+	if (da->last_hit >= last_hit) last_hit = da->last_hit;*/
+	       	
+	if (db->last_hit >= last_hit && da->last_hit < last_hit){
+		return 1;
+	}
+	if (da->last_hit >= last_hit && db->last_hit < last_hit){
+		return -1;
+	}
+	if (db->hits != da->hits) return (db->hits - da->hits);
+	/*if (db->last_hit != da->last_hit) return db->last_hit - da->last_hit;*/
 	return (strcmp(da->path,db->path));
 }
 
+static void history_lasthit(DBHashTable * d)
+{
+    history_dbh_t *history_mem = (history_dbh_t *)DBH_DATA(d);
+    if (!history_mem) g_assert_not_reached();
+    if (history_mem->last_hit >= last_hit) {
+		last_hit=history_mem->last_hit;
+    }
+}
+
 static void history_mklist(DBHashTable * d)
 {
     history_dbh_t *history_mem = (history_dbh_t *)malloc(sizeof(history_dbh_t));
     if (!history_mem) g_assert_not_reached();
     memcpy(history_mem,DBH_DATA(d),sizeof(history_dbh_t));
     if (!the_list) g_assert_not_reached(); 
-    *the_list = g_list_insert_sorted(*the_list,history_mem,history_compare);
+    if (history_mem->path && strlen(history_mem->path))
+       *the_list = g_list_insert_sorted(*the_list,history_mem,history_compare);
 }
 
 
-static void clean_history_list(GList **list){
-	GList *tmp;
-	if (!*list) return ;
-	for (tmp=*list;tmp;tmp=tmp->next){
-		/*printf("freeing %s\n",(char *)tmp->data);*/
-		g_free(tmp->data);
-	}
-	g_list_free(*list);
-	*list=NULL;
-	return;
-}
 
 /* if top==NULL, the top entry is left blank,
  * if top=="", the top entry is the one with the greatest access time for last hit
  * if top=="anything", the entry is set to "anything"
  * */
-gchar *active_dbh_file=NULL;
 
 void get_history_list(GList **in_list,char *dbh_file,char *top){
    DBHashTable *d;
@@ -1067,11 +1461,12 @@ void get_history_list(GList **in_list,char *dbh_file,char *top){
    char *first=NULL;
    time_t last_hit=0;
    the_list=in_list;
-   g_free(active_dbh_file);
-   active_dbh_file=g_strdup(dbh_file);
+
 	   
-   clean_history_list(the_list);	
+   clean_history_list(the_list);
+   last_hit=0;   
    if ((d=DBH_open(dbh_file))!=NULL){
+	DBH_foreach_sweep(d,history_lasthit);	
 	DBH_foreach_sweep(d,history_mklist);	
 	DBH_close(d);
    }
@@ -1088,6 +1483,7 @@ void get_history_list(GList **in_list,char *dbh_file,char *top){
 #endif
 	tmp->data=p;
 	g_free(history_mem);
+	history_mem=NULL;
    }
    
    
@@ -1130,6 +1526,24 @@ void save_to_history(char *dbh_file,char *path2save){
     	DBH_close(d);
 }
 
+gboolean is_in_history(char *dbh_file,char *path2save){
+    	GString *gs;
+	DBHashTable *d;
+	history_dbh_t *history_dbh;
+	gboolean found=FALSE;
+	
+	if (strlen(path2save) > 255) return;
+	if ((d=DBH_open(dbh_file))==NULL) return FALSE;
+	gs = g_string_new(path2save);
+    	sprintf((char *)DBH_KEY(d), "%10u", g_string_hash(gs));
+   	g_string_free(gs, TRUE);
+
+	history_dbh = (history_dbh_t *)DBH_DATA(d);
+	if (DBH_load(d)) found=TRUE;
+    	DBH_close(d);
+	return found;
+}
+
 void remove_from_history(char *dbh_file,char *path2save){
     	GString *gs;
 	DBHashTable *d;
