@@ -47,6 +47,7 @@
 #include "add_node_contents.h"
 #include "aux.h"
 #include "bookmarks.h"
+#include "dnd.h"
 #include "dummies.h"
 #include "entry.h"
 #include "filter.h"
@@ -70,12 +71,21 @@
 /* bug workaround (double click signals click first)*/
 static gboolean skip = FALSE;
 static GtkTreeRowReference *title_reference=NULL;
+static mousedown=FALSE;
 
 extern int stop;
 /*XXX for gtk bug workaround with flashing treeview on expand/collapse
  * ater scroll if root not visible: */
 extern GtkTreeViewColumn *name_column;
 
+static gboolean valid_iter(GtkTreeView * treeview,GtkTreeIter * iter){
+    tree_entry_t *en;
+    GtkTreeModel *treemodel = gtk_tree_view_get_model(treeview);
+    gtk_tree_model_get(GTK_TREE_MODEL(treemodel), iter, ENTRY_COLUMN, &en, -1);
+    if (!en) return FALSE;
+    if (IS_PATH(en->type) && access(en->path,F_OK)!=0) return FALSE;
+    return TRUE;
+}
 
 void open_dir(GtkTreeView * treeview, GtkTreeIter * iter, GtkTreePath * treepath, gpointer user_data)
 {
@@ -91,10 +101,17 @@ void open_dir(GtkTreeView * treeview, GtkTreeIter * iter, GtkTreePath * treepath
 #ifdef DEBUG
 	printf("DBG: open_dir ...\n");
 #endif
+    mousedown=FALSE; /* gtk bug workaround:click on expander passes button_press but not button_release */
+
+    if (!valid_iter(treeview,iter)){
+	    local_monitor(treeview,TRUE);
+	    return;
+    }
    if (!window_title) {
 	    window_title=(gchar **)malloc(sizeof(gchar *));
 	    *window_title=NULL;
     }
+  
     if(!set_load_wait(&tree_details))
     {
 	gtk_tree_view_collapse_row(treeview, treepath);
@@ -128,7 +145,7 @@ void open_dir(GtkTreeView * treeview, GtkTreeIter * iter, GtkTreePath * treepath
 
     if(stat(en->path, &st) >= 0)
     {
-	if(st.st_mtime != en->st->st_mtime)
+	if(st.st_mtime != en->st->st_mtime || st.st_ctime != en->st->st_ctime)
 	{
 	    /*printf("DBG: UNSET_LOADED st.st_mtime != en->st->st_mtime\n");*/
 	    memcpy(en->st, &st, sizeof(struct stat));
@@ -357,6 +374,7 @@ void open_dir(GtkTreeView * treeview, GtkTreeIter * iter, GtkTreePath * treepath
 #endif
     if(en && en->path){
 	g_free(*window_title);
+	*window_title=NULL;
 	 /* ascii readable conversion for smb items are here */
 	if (title_reference) gtk_tree_row_reference_free(title_reference);
        	title_reference = gtk_tree_row_reference_new(treemodel, treepath);
@@ -399,6 +417,12 @@ void close_dir(GtkTreeView * treeview, GtkTreeIter * iter, GtkTreePath * path, g
     GtkTreeModel *treemodel = gtk_tree_view_get_model(treeview);
     tree_details_t *tree_details = get_tree_details(treeview);
 
+    mousedown=FALSE; /* gtk bug workaround:click on expander passes button_press but not button_release */
+
+    if (!valid_iter(treeview,iter)){
+	    local_monitor(treeview,TRUE);
+	    return;
+    }
     if(tree_details->loading)
     {
 	/* this creates an endless loop: 
@@ -453,6 +477,7 @@ void close_dir(GtkTreeView * treeview, GtkTreeIter * iter, GtkTreePath * path, g
     if(en->path)
     {
 	g_free(*window_title);
+	*window_title=NULL;
 #if 0
 	if(en->count > 512)
 	{
@@ -492,6 +517,19 @@ void close_dir(GtkTreeView * treeview, GtkTreeIter * iter, GtkTreePath * path, g
 
 /********************* callbacks **************************/
 
+gboolean on_mouse_move(GtkWidget *widget,GdkEventMotion *event, gpointer user_data)
+{
+	static gboolean reselected=FALSE;
+	if (!mousedown) {
+		reselected=FALSE;
+	}
+	else if (!reselected) {
+		reselected=TRUE;
+ 		reselect_dnd_list((GtkTreeView *)widget);
+	}
+	return FALSE;
+}
+	
 gboolean button_releaseF(GtkWidget * widget, GdkEventButton * event, gpointer data)
 {
     GtkTreeView *treeview = (GtkTreeView *) data;
@@ -501,7 +539,7 @@ gboolean button_releaseF(GtkWidget * widget, GdkEventButton * event, gpointer da
     tree_entry_t *en;
     GtkTreeIter iter;
 
-    
+   mousedown=FALSE; 
     if(tree_details->loading)	return FALSE;
 
    if(event->button == 1 && skip) 
@@ -533,12 +571,13 @@ gboolean button_releaseF(GtkWidget * widget, GdkEventButton * event, gpointer da
 				readable_path,
 				NULL);
 		g_free(readable_path);
+		readable_path=NULL;
 		set_path_reference(treeview,treepath);
 		
 		gtk_tree_path_free(treepath);
 		turn_on(treeview);
 	    }
-	    
+	    clear_dnd_selection_list();
 	    break;
 	case 2:
 	    if(gtk_tree_view_get_path_at_pos(treeview, event->x, event->y, &treepath, NULL, NULL, NULL))
@@ -566,10 +605,12 @@ gboolean button_releaseF(GtkWidget * widget, GdkEventButton * event, gpointer da
 		    print_diagnostics(treeview, icono, 
 				readable_path, "\n", NULL);
 		    g_free(readable_path);
+		    readable_path=NULL;
 		}
 
 		gtk_tree_path_free(treepath);
 	    }
+	    clear_dnd_selection_list();
 	    break;
     }
     return FALSE;
@@ -590,6 +631,7 @@ gboolean treeclick(GtkWidget * widget, GdkEventButton * event, gpointer data)
     gchar *newpath=NULL;
     int type;
 
+    mousedown=TRUE;
     if(tree_details->loading)
 	return TRUE;
 
@@ -655,8 +697,8 @@ gboolean treeclick(GtkWidget * widget, GdkEventButton * event, gpointer data)
 		    	gtk_tree_selection_unselect_path (selection,treepath);
 		    	gtk_tree_path_free(treepath);
 		    }
-		    /*go_to(treeview, newpath);*/
-		    go_to(tree_details->treeview, newpath);
+		    go_to(treeview, newpath);
+		    /*go_to(tree_details->treeview, newpath);*/
 		}
 
 		else if(IS_EXE(en->type) )
@@ -683,6 +725,7 @@ gboolean treeclick(GtkWidget * widget, GdkEventButton * event, gpointer data)
 	    }
 done:
 	    g_free(newpath);
+	    newpath=NULL;
 	}
 	gtk_tree_selection_set_mode(selection, GTK_SELECTION_MULTIPLE);
 	return TRUE;
@@ -693,12 +736,21 @@ done:
 	default:
 	    break;
 	case 3:
+	    clear_dnd_selection_list(); /* only currently selected items */
 	    if(gtk_tree_view_get_path_at_pos(treeview, event->x, 
 				    event->y, &treepath, 
-				    NULL, NULL, NULL)){
+				    NULL, NULL, NULL))
+	    {
 		gtk_tree_selection_select_path(selection,treepath);    
-       		gtk_tree_view_set_cursor (treeview,treepath,NULL,FALSE);
-		gtk_tree_path_free(treepath);
+		gtk_tree_model_get_iter (treemodel,&iter,treepath); 
+	        gtk_tree_model_get(treemodel, &iter, ENTRY_COLUMN, &en, -1);
+		if (!IS_PATH(en->type) && ! IS_NETTHING(en->subtype) ) {
+		    /*this unselects other stuff: */
+		    gtk_tree_view_set_cursor (treeview,treepath,NULL,FALSE);
+		} else {
+    	           gtk_tree_selection_selected_foreach(selection, get_dnd_selection, (gpointer) treeview);
+		}
+	       	gtk_tree_path_free(treepath);
 	    }
 	    do_popup(treeview, event);
 	    return TRUE;
@@ -710,6 +762,20 @@ done:
 	    /* this enables dnd(right), but also selection(wrong)*/
 #endif
 	case 1:
+	  if (event->state!=GDK_CONTROL_MASK && event->state!=GDK_SHIFT_MASK) {
+	    clear_dnd_selection_list();
+    	    gtk_tree_selection_selected_foreach(selection, get_dnd_selection, (gpointer) treeview);
+	    if(gtk_tree_view_get_path_at_pos(treeview, event->x, 
+				    event->y, &treepath, 
+				    NULL, NULL, NULL))	
+	    if (!is_in_dnd_selection(treepath)){
+		if (gtk_tree_model_get_iter (treemodel,&iter,treepath)) 
+		{
+		  get_dnd_selection(treemodel, treepath, &iter, (gpointer) treeview);
+		}
+		gtk_tree_path_free(treepath);
+	    }
+	  }
 	    return FALSE;
     }
     return FALSE;
