@@ -1,4 +1,4 @@
-/*#define DEBUG*/
+/*#define ENHANCED_CODE*/
 /*
  * Copyright (C) 2002 Edscott Wilson Garcia
  * EMail: edscott@imp.mx
@@ -165,7 +165,7 @@ static void status_the_path(GtkTreeView *treeview, GtkTreePath *ipath,tree_entry
 		print_status_tmp(treeview, resolve_icon_small(en), 
 				readable_path,
 				NULL);
-		g_free(readable_path);
+		g_free(readable_path);readable_path=NULL;
 		turn_on(treeview);
 	    }
 
@@ -178,9 +178,12 @@ static gboolean null_motion(GtkWidget * widget, GdkDragContext * dc, gint x, gin
 
 static void create_menu_entry(char *name,char *label,GtkWidget *parent,
 		GtkWidget *main_window,void *callback,void *data){
-  GtkWidget *w;
-  w = gtk_menu_item_new_with_label (label);
+  GtkWidget *w,*image;
+  w = gtk_image_menu_item_new_with_label (label);
   gtk_container_add (GTK_CONTAINER (parent), w);
+  image = gtk_image_new_from_stock ("gtk-execute", GTK_ICON_SIZE_MENU);
+  gtk_widget_show (image);
+  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (w), image);
   g_signal_connect ((gpointer) w, "activate",
                     G_CALLBACK (callback),
                    data);
@@ -484,7 +487,7 @@ void create_root_element(GtkTreeView * treeview, GtkTreeIter *iterator,
 	    default:
 		break;
 	}
-	g_free(tag);
+	g_free(tag);tag=NULL;
 }
 
 static void add_roots(GtkTreeView * treeview, char *home)
@@ -530,16 +533,6 @@ static void add_roots(GtkTreeView * treeview, char *home)
     return;
 }
 
-static void count_selection(GtkTreeModel * treemodel, 
-		GtkTreePath * treepath, 
-		GtkTreeIter * iter, 
-		gpointer data)
-{
-    GtkTreeView *treeview = (GtkTreeView *) data;
-    tree_details_t *tree_details = get_tree_details(treeview);
-    tree_details->selectionOK++;
-    return ;
-}
 
 
 /* additional key bindings for the tree */
@@ -662,16 +655,20 @@ static gboolean treeview_key(GtkWidget *w,
 					
 			gtk_widget_grab_focus((GtkWidget *) treeview2);
     		  	gtk_tree_view_set_cursor (treeview2,ipath,NULL,FALSE);
+    			clear_dnd_selection_list();
 			gtk_tree_selection_select_path(selection2,ipath);
 			
 			gtk_tree_model_get_iter(treemodel, &iter, ipath);
+		        get_dnd_selection(treemodel, ipath, &iter, (gpointer) treeview2);
+			
 			gtk_tree_model_get(treemodel, &iter, ENTRY_COLUMN, &en, -1);
 
 			if (en) status_the_path(treeview,ipath,en);
 			
 			gtk_tree_path_free(ipath);
 		        lock=FALSE;
-	 
+			turn_on(treeview);
+ 
 			return TRUE;
 		}
 		case GDK_Menu:
@@ -695,6 +692,7 @@ static gboolean treeview_key(GtkWidget *w,
 			return TRUE;
 		}
 		case GDK_Escape:
+    			clear_dnd_selection_list();
 			on_unselect_activate((GtkMenuItem *)w,NULL);
 			return TRUE;
 		default: break;
@@ -706,6 +704,7 @@ static gboolean treeview_key(GtkWidget *w,
 	if(gtk_tree_selection_get_selected(selection, &treemodel, &iter))
   		   gtk_tree_model_get(treemodel, &iter, ENTRY_COLUMN, &en, -1);
 	gtk_tree_selection_set_mode(selection, GTK_SELECTION_MULTIPLE);
+	
 
 	if ((event->keyval >= GDK_space && event->keyval <= GDK_asciitilde) || 
 	    (event->keyval >= GDK_KP_Multiply && event->keyval <= GDK_KP_9)  
@@ -801,7 +800,9 @@ static gboolean treeview_key(GtkWidget *w,
 			gdk_flush();
     			gtk_tree_view_scroll_to_cell(treeview, ipath, 
 					NULL, TRUE, 0.0, 0.0);
+    			clear_dnd_selection_list();
 			gtk_tree_selection_select_path(selection,ipath);
+		        get_dnd_selection(treemodel, ipath, &iter, (gpointer) treeview);
        			gtk_tree_view_set_cursor (treeview,ipath,NULL,FALSE);
 			if (en) status_the_path(treeview,ipath,en);	
 			set_path_reference (treeview,ipath);
@@ -821,7 +822,7 @@ static gboolean treeview_key(GtkWidget *w,
 	    }
 	    g_free(a);
 	    g_free(b);
-	    
+	    a=b=NULL;
     	    if (ipath) gtk_tree_path_free(ipath);
 	    return FALSE;
 	}	    
@@ -882,9 +883,12 @@ static gboolean treeview_key(GtkWidget *w,
 			set_path_reference(treeview,ipath);
 		    	gtk_widget_grab_focus((GtkWidget *) treeview);
     		  	gtk_tree_view_set_cursor (treeview,ipath,NULL,FALSE);
+    			clear_dnd_selection_list();
 			gtk_tree_selection_select_path(selection,ipath);
 			
 			gtk_tree_model_get_iter(treemodel, &iter, ipath);
+		        get_dnd_selection(treemodel, ipath, &iter, (gpointer) treeview);
+			
 			gtk_tree_model_get(treemodel, &iter, ENTRY_COLUMN, &en, -1);
 
 			if (en) status_the_path(treeview,ipath,en);
@@ -893,6 +897,8 @@ static gboolean treeview_key(GtkWidget *w,
 				/*get selected path ++*/
 			     gdk_flush();
 			     gtk_tree_view_scroll_to_cell(treeview, ipath, NULL, TRUE, 0.0, 0.0);
+			     clear_dnd_selection_list();
+			     get_dnd_selection(treemodel, ipath, &iter, (gpointer) treeview);
 			}
 			
 			gtk_tree_path_free(ipath);
@@ -959,13 +965,44 @@ static gboolean treeview_key(GtkWidget *w,
 			printf("DBG: lock #3!\n");
 #endif			
 			
+#ifdef ENHANCED_CODE
+			{	
+			  GtkTreeIter parent;
+			  GtkTreePath *tpath=gtk_tree_model_get_path(treemodel, &iter);
+			  
+			  if (gtk_tree_view_row_expanded (treeview,tpath)){
+			     gtk_tree_view_collapse_row(treeview, tpath);
+		    	     gtk_tree_path_free(tpath);
+			     turn_on(treeview);
+			     lock=FALSE;
+			     return TRUE;
+			  } 
+			  if (gtk_tree_model_iter_parent (treemodel,&parent,&iter)){
+		    	     gtk_tree_path_free(tpath);
+			     tpath=gtk_tree_model_get_path(treemodel, &parent);
+			     gtk_tree_view_collapse_row(treeview, tpath);
+				     
+    			     gtk_tree_view_scroll_to_cell(treeview, tpath, NULL, TRUE, 0.0, 0.0);
+
+			     clear_dnd_selection_list();
+
+			     gtk_tree_selection_select_path (selection,tpath);
+			     gtk_tree_view_set_cursor (treeview,tpath,NULL,FALSE);
+		             get_dnd_selection(treemodel, tpath, &parent, (gpointer) treeview);
+
+		    	     gtk_tree_path_free(tpath);
+			     turn_on(treeview);
+			     lock=FALSE;
+			     return TRUE;
+			  }  
+			}
+			
+#else
+			
 			if (!gtk_tree_model_iter_has_child(treemodel,&iter)){
 			 lock=FALSE;
 			 return FALSE;
-			}
-
-	
-			 
+			}	 
 			if (gtk_tree_model_iter_has_child(treemodel,&iter))
 			{
     			     GtkTreePath *treepath;
@@ -974,15 +1011,9 @@ static gboolean treeview_key(GtkWidget *w,
 		    	     gtk_tree_path_free(treepath);
 			     turn_on(treeview);
 			} 
-#if 0
-  /* as above, this is left to return on the .. element */
-			else {
-#ifdef DEBUG	
-			printf("DBG: 3 >>> tb_go_up!\n");
-#endif			
-				tb_go_up((GtkButton *)w,NULL);
-			}
+			
 #endif
+			
 			lock=FALSE;
 #ifdef DEBUG	
 			printf("DBG: unlock #3!\n");
@@ -1043,6 +1074,7 @@ GtkTreeView *create_treeview(char *path)
 
     if((treestore = create_treestore()) == NULL)
 	g_assert_not_reached();
+    /*printf("DBG: GTK_TREE_STORE (tree_model)->n_columns=%d\n",GTK_TREE_STORE (treestore)->n_columns);*/
 
     if((treeview = (GtkTreeView *) gtk_tree_view_new_with_model(GTK_TREE_MODEL(treestore))) == NULL)
 	g_assert_not_reached();
@@ -1056,6 +1088,7 @@ GtkTreeView *create_treeview(char *path)
     tree_details->timer = 0;
     tree_details->remove = NULL;
     tree_details->loading = 0;
+    tree_details->input = OTHER_INPUT;
     tree_details->tubo_object = NULL;
     tree_details->theme = g_strdup("gnome");
 
@@ -1150,6 +1183,8 @@ GtkTreeView *create_treeview(char *path)
     cell = gtk_cell_renderer_text_new();
     g_object_set(G_OBJECT(cell), "editable", FALSE, NULL);
     gtk_tree_view_column_set_title(column, my_utf_string(_("Name")));
+    gtk_tree_view_column_set_clickable(column, TRUE);
+    g_signal_connect_object(column, "clicked", G_CALLBACK(on_name_click), treeview, 0);
 
 
 #ifdef SPLIT_VIEW
@@ -1189,8 +1224,6 @@ GtkTreeView *create_treeview(char *path)
 	gtk_tree_view_column_set_title(column, s);
 	g_free(s);
     }*/
-    gtk_tree_view_column_set_clickable(column, TRUE);
-    g_signal_connect_object(column, "clicked", G_CALLBACK(on_name_click), treeview, 0);
 
 
     for(i = SIZE_COLUMN, j = SIZE_COL; i < TREE_COLUMNS; i++, j++)
@@ -1233,8 +1266,9 @@ GtkTreeView *create_treeview(char *path)
 	    default:
 		texto = NULL;
 	}
-	gtk_tree_view_column_set_attributes(column, cell, "style", STYLE_COLUMN, 
-			"font-desc", FONT_COLUMN, "text", i, NULL);
+	gtk_tree_view_column_set_attributes(column, cell,  
+			"font-desc", FONT_COLUMN, 
+			"text", i, NULL);
 	
 	/* translated texts should already be in UTF-8 format 
 	 * (in Leibnitz' world) */
@@ -1260,6 +1294,7 @@ GtkTreeView *create_treeview(char *path)
     g_signal_connect_object(treeview, "row-collapsed", G_CALLBACK(close_dir), treeview, 0);
 
 
+    g_signal_connect(treeview, "motion-notify-event", G_CALLBACK(on_mouse_move), treeview);
     g_signal_connect(treeview, "button-press-event", G_CALLBACK(treeclick), treeview);
 /* g_signal_connect works better than g_signal_connect_object, sometimes */
     g_signal_connect_object(treeview, "button-release-event", 
@@ -1275,6 +1310,7 @@ GtkTreeView *create_treeview(char *path)
     g_signal_connect_object(treeview2, "row-collapsed", G_CALLBACK(close_dir), treeview2, 0);
 
 
+    g_signal_connect(treeview2, "motion-notify-event", G_CALLBACK(on_mouse_move), treeview2);
     g_signal_connect(treeview2, "button-press-event", G_CALLBACK(treeclick), treeview2);
 /* g_signal_connect works better than g_signal_connect_object, sometimes */
     g_signal_connect_object(treeview2, "button-release-event", 
@@ -1397,7 +1433,7 @@ GtkTreeView *create_treeview(char *path)
     	  label = gtk_bin_get_child(GTK_BIN(a));
 	  label_txt=g_strconcat(_("About")," (",VERSION,")",NULL);
 	  gtk_label_set_text((GtkLabel *)label,my_utf_string(label_txt));
-	  g_free(label_txt);
+	  g_free(label_txt);label_txt=NULL;
     }
 
 
@@ -1443,7 +1479,7 @@ GtkTreeView *create_treeview(char *path)
     g_signal_connect(treeview, "key-press-event", 
 			G_CALLBACK(treeview_key),
                         NULL);    
-    g_free(d_path);
+    g_free(d_path);d_path=NULL;
     gtk_widget_grab_focus((GtkWidget *) treeview);
     set_filter_combo(treeview);
     
@@ -1473,8 +1509,7 @@ GtkTreeView *init_xffm(int argc, char *argv[])
 	 strstr(xffm_argv0,"xfapps") ||    
 	 strstr(xffm_argv0,"xftree") ||    
 	 strstr(xffm_argv0,"xfglob") ||    
-	 strstr(xffm_argv0,"xffstab") ) && 
-	argc < 2 ) 
+	 strstr(xffm_argv0,"xffstab") ) ) 
 	    expand=TRUE;
     if((strstr(xffm_argv0,"xffm") || strstr(xffm_argv0,"xftree"))&& argc >= 2)
     {				/* if it fails it goes to root. */
@@ -1493,7 +1528,7 @@ GtkTreeView *init_xffm(int argc, char *argv[])
 
     reg_build_list();
     treeview = create_treeview(startup);
-    g_free(startup);
+    g_free(startup);startup=NULL;
     if (expand){
 	GtkTreeIter iter;
 	tree_entry_t *en=NULL;
@@ -1520,7 +1555,9 @@ GtkTreeView *init_xffm(int argc, char *argv[])
     	   gtk_tree_view_expand_row(treeview, treepath, FALSE);  
      	   gdk_flush();
 	   gtk_tree_view_scroll_to_cell(treeview, treepath, NULL, TRUE, 0.0, 0.0);
+    	   clear_dnd_selection_list();
 	   gtk_tree_selection_select_path (selection,treepath);
+           get_dnd_selection(treemodel, treepath, &iter, (gpointer) treeview);	   
 	   gtk_tree_view_set_cursor (treeview,treepath,NULL,FALSE);
  	   gtk_tree_path_free(treepath); 
 	}
